% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [x] = AddNewConsequent(x, j, i, data_window)
%% Description
% This function obtains the consequent parameters of the new fuzzy rule (Equation (23))

%% Function inputs
% x - Struct with all fuzzy control parameters
% j - input variable, in whichwas added a the new fuzzy rule
% i - index of the new fuzzy rule 
% data_window - temporal sliding window

%% Function output
% x - Struct with all fuzzy control parameters

%% Auxiliary variables
window_size = size(data_window, 2); % size of data_window

%%  Consequent parameters of the new fuzzy rule  - equation (23)

num = 0.0; % Auxiliary variable
den = 0.0; % Auxiliary variable

if j == 1
    var = size(data_window, 1)-1; %if the selected variable is ref (j=1), this variable is stored at end-1 at data_window
else
    var = j;
end

for m=1:window_size% for all temporal sliding window
    
    % MF parameters of the fuzzy rule i of input variable j
    parameters(1,1) = x(j).Rules(i,1); % lower limit 
    parameters(1,2) = x(j).Rules(i,2); % center
    parameters(1,3) = x(j).Rules(i,3); % upper limit
    
    mu_varialble = trifp(data_window(var,m), parameters); % fuzzy degree on the MF of the fuzzy rule i of input variable j
    
    num = num + data_window(end,m) * mu_varialble;% numerator of equation (23)
    den = den + mu_varialble; % denominator of equation (23)
        
end

x(j).Rules(i,4) = num/den;% equation (23)

end