function out = CSTR(u, k)
% Paper: Design of nonlinear PID controller and nonlinear model predictive controller for a continuous stirred tank reactor
% de: J. Prakash , K. Srinivasan
% em: ISA Transactions


% Ca -- Concentration (mol/l)
% T -- Temperature (K)
% qc -- Coolant flow rate (l/min)
% q -- Feed flow rate (l/min)
% Cao -- Feed concentration (mol/l)
% T0 -- Feed temperature (K)
% Tc0 -- Inlet coolant temperature (K)
% V -- CSTR volume (l)
% hA -- Heat transfer term (cal/(min K))
% k0 -- Reaction rate constant (min−1 )
% E_R -- Activation energy term (K)
% DeltaH -- Heat of reaction (cal/mol)
% rho rhoc -- Liquid density (g/l)
% Cp Cpc -- Specific heats (cal/(g K))
% h -- amostragem (min)

global Ca T qc


q = 100;
Cao = 1;
T0 = 350;
Tc0 = 350;
V = 100;
hA = 7e5;
k0 = 7.2e10;
E_R = 1e4;
DeltaH = 2e5;
rho = 1e3;
rhoc = 1e3;
Cp = 1; 
Cpc = 1; 
h = 0.1;
d=ceil(0.5/h);
qc(k) = u;


dCa(k) = q/V*(Cao-Ca(k-1)) - k0*Ca(k-1)*exp(-E_R*(1/T(k-1-d)));

dT = q/V*(T0-T(k-1-d)) + (DeltaH*k0*Ca(k-1))/(rho*Cp)*exp(-E_R*(1/T(k-1-d))) + (rhoc*Cpc)/(rho*Cp*V)*qc(k)*(1-exp(-hA/(qc(k-d)*rhoc*Cpc)))*(Tc0-T(k-1-d));

Ca(k) = Ca(k-1) + h*dCa(k);

T(k-d) = T(k-1-d) + h*dT;

% disp('****************')
% bb(k)=(DeltaH)*k0*Ca(k-1)/(rho*Cp)*exp(-E_R*(1/T(k-1-d)));
% cc(k)=(rhoc*Cpc)/(rho*Cp*V)*qc(k)*(1-exp(-hA/(qc(k-d)*rhoc*Cpc)))*(Tc0-T(k-1-d));

% pause
out = Ca(k);

end