% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [ ] = Plot_Evolve_Parameterts (Evolve_Parameterts)
%% Description
% This function does plots of some important parameters on the evolving stage

%% Function inputs
% Evolve_Parameterts - Struct with some importante parameters on the evolving stage to do plots

%% Function output
% several plots

%% Flag Add
figure
plot(Evolve_Parameterts.Add) % Plot of the flag Add - indicates when new fuzzy rule was added
labels{1} = 'New rule was added'; % Save legend

%% Number of rules per input variable
for j=1:size(Evolve_Parameterts.nRules,2)
    hold on
    plot(Evolve_Parameterts.nRules{j})  
    labels{j+1} = strcat('Number of rules of x_', num2str(j)); % Save legend
    
end
legend(labels) % Figure legends
title('Evolution of the fuzzy rules') % Figure title

%% Evolution of Criterion 1
figure
plot(Evolve_Parameterts.delta_dif_controller)
title('Evolution of Criterion 1') % Figure title

%% Evolution of the center of the antecedents membership functions and of the consequents
kk=1;
kkk=1;
for j=1:size(Evolve_Parameterts.nRules,2)
    
    % change of variables
    for k=1:size(Evolve_Parameterts.MFs,2)
        aux= Evolve_Parameterts.MFs{j,k,:};
        for i=1:size(aux,1)
            antecedents(i,k) = aux(i,2);
            consequents(i,k) = aux(i,4);
        end
        
    end
    
    % Plot of the center of the antecedents membership functions
    figure
    for i=1:size(antecedents,1)
        hold on
        plot(antecedents(i,:))
        labels{i} = strcat('MF  ', num2str(i)); % Save legend
        kk=kk+1;
    end
    legend(labels) % Figure legends
    title(sprintf('Evolution of the antecendents of x_{%d}', j)) % Figure title

    % Plots of the consequents
    figure
    for i=1:size(consequents,1)
        hold on
        plot(consequents(i,:))
        labels{i} = strcat('Rule ', num2str(i)); % Save legend
        kkk=kkk+1;
    end
    legend(labels) % Figure legends
    title(sprintf('Evolution of the consequents of x_{%d}', j)) % Figure title
end

end
