% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [x] = UpdateAntecedent( x, data_input)
%% Description
% This function updates the antecedent parameters (Algorithm 2 Step 3.a)

%% Function inputs
% x - Struct with all fuzzy control parameters
% data_input -  values of the input variable

%% Function output
% x - Struct with all fuzzy control parameters

%% Global Variables
global beta

%% Variables Definition
n_var= size(x, 2); % number of inputs variables

%% Obtain variable \psi (eq. 11)
psi = Psi( data_input, x);

%% Antecedents update

k = 1; % Auxiliary variable

for j = 1:n_var % for all input variables
    
    n_rules = size( x(j).Rules,1 ); % number of rules of input variable j

    for i = 2:n_rules-1 % for all MFs of input variable j, except the first and last ones
        
        parameters(1,1) = x(j).Rules(i,1); % parameter a_{j,i}
        parameters(1,2) = x(j).Rules(i,2); % parameter b_{j,i}
        parameters(1,3) = x(j).Rules(i,3); % parameter c_{j,i}

        x(j).Rules(i,2) = x(j).Rules(i,2) + beta*psi(k)*( data_input(j) -  x(j).Rules(i,2) ); % adapt the center of MF i
        x(j).Rules(i-1,3) = x(j).Rules(i,2); % update the parameter c_{j,i} of MF i-1 (left MF)
        x(j).Rules(i+1,1) = x(j).Rules(i,2); % update the parameter a_{j,i} of MF i+1 (right MF)
 
        k = k +1;
    end   
end

end