% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [x] = UpdateConsequent(x, C, error, data_input, control_min_max)
%% Description
% This function updates the consequent parameters (Algorithm 2 Step 3.b)

%% Function inputs
% x - Struct with all fuzzy control parameters
% C - is the adaptation gain 
% error - is the tracking error, i.e. r(k)-y(k)
% data_input -  values of the input variable
% control_min_max - Universe of discourse of the control variable [min max]

%% Function output
% x - Struct with all fuzzy control parameters

%% Variables Definition
n_var= size(x, 2); % number of inputs variables

%% Obtain variable \psi (eq. 11)
psi = Psi(data_input, x);

%% Consequents update

k = 1; % Auxiliary variable

for j=1:n_var % for all input variables
    
    n_rules = size( x(j).Rules,1 ); % number of rules of input variable j
        
    for i = 1:n_rules % for all rules of input variable j,

        x(j).Rules(i,4) = x(j).Rules(i,4) + C*psi(k)*error;  % adapt the consequent of rule i
        
        
        if x(j).Rules(i,4) < control_min_max(1)
            x(j).Rules(i,4)  = control_min_max(1);  % Limitation of the consequent of rule i by the minimal control value admissible
         end
        
        if x(j).Rules(i,4) > control_min_max(2)
            x(j).Rules(i,4) = control_min_max(2); % Limitation of the consequent of rule i by the maximal control value admissible
    
        end
        
        k = k +1;
    end
    
end


end