% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function u_mf = trifp(data_mf, parameters)
%% Description
% This function obtains the fuzzy degree of a triangular MF for a given data

%% Function inputs
% data_mf - data
% parameters - parameters of the triangular MF - parameters = [a b c] which
% are,  the lower limit, center value, and upper limit of the MF,
% respectively.

%% Function output
% data_mf - fuzzy degree of a triangular MF

u_mf = 0;

if (data_mf> parameters(1) && data_mf< parameters(2)) % left size
    u_mf = (data_mf-parameters(1))/(parameters(2)-parameters(1));      
elseif (data_mf>= parameters(2) && data_mf< parameters(3))% right size
    u_mf = (parameters(3)-data_mf)/(parameters(3)-parameters(2));
elseif (data_mf<= parameters(1) || data_mf>= parameters(3)) % outside
    u_mf = 0;
end

end