% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

%% Clean everything
clc
clear all
close all

%% add path
addpath(genpath('./Functions'));

%% configuration file
ParametersConfig % here are the variables that must be definded by the user (Algorithm 1 Step 1)

%% Definition of the reference and Plant variables
reference

%% Fuzzy Rules Design Initialization - Algorithm 2 Step 2
x = Initialization( universe_x, Rules_ini, eta_aux, control_min_max(1) ); % Definition/design of the struct that contain all parameters;

%% Global variables
global beta dif_controller Evolve_Parameterts delta delta_dif_controller

%% Variables Definition
data_window = []; % temporal sliding window
dif_controller = [];% the difference of the estimated control error
delta_dif_controller  = [];% the difference of difference of the estimated control error

%% Save some importante parameters to do plots 
Evolve_Parameterts.Add(1) = 0;
Evolve_Parameterts.Sample = max_delay; % usually is 1, however in this plant there is a delay
for j=1:size(x,2)% for all input variables
    Evolve_Parameterts.nRules{j} = Rules_ini(j);% number of rules
    Evolve_Parameterts.MFs{j,Evolve_Parameterts.Sample} = x(j).Rules; % rules matrix (parameters)
end

%% Control cycle
for i=max_delay:ref_end
    
    % just to print the sample time
    if mod(i,1000) == 0
        i
    end
    
    %% Inputs variables
    error = rk(i)-yk(i-1); %input x1
    data_inputs = [rk(i) yk(i-1)];%input x2
    
    %% Evolving Controller
    [uk(i), x] = EvolveFLC(data_inputs, x, C, error, data_window, window_size, control_min_max);
        
    %% Plant to be controlled - It is just an example to allow use this toolbox
    yk(i) = CSTR(uk(i-1),i);

    %% update_window
    if i ~= max_delay
        data_window = update_window( data_window, yk(i), uk(i), data_inputs, window_size);
    end
    
end

%% Final Plots
PlotResults % reference; output; and command signal

Plot_Evolve_Parameterts (Evolve_Parameterts)% plots of some importante parameters on the evolving stage

PlotMFs(x,10000)% Plots of the final membership functions of all input variables



