% For use with Matlab
% Copyright (C) 2019 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Francisco Souza, and Rui Araujo. "Online Evolving Fuzzy Control Design: An Application to a CSTR Plant". In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), 2017.
% The "SEDFLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Francisco Souza and Rui Araujo. 
% Online Evolving Fuzzy Control Design: An Application to a CSTR Plant.
% In Proc. IEEE 15th International Conference on Industrial Informatics (INDIN 2017), pages 1-8, Emden, Germany, July 24 - July 17
% DOI: https://doi.org/10.1109/INDIN.2017.8104774
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [error_controller_estimation, error_controller_estimation_local] = FuzzyController_EstimationError (x, data_window, window_size)
%% Description
% This function obtains the estimated control error (equation (19), see Section III-C)

%% Function inputs
% x - Struct with all fuzzy control parameters
% data_window - temporal sliding window
% window_size - Size of the temporal sliding window 

%% Function output
% error_controller_estimation - estimated control error, equation (19)
% error_controller_estimation_local - estimated control error for each rule

%% Variables Definition
n_var= size(x, 2); % number of inputs variables
error_controller_estimation_local = zeros(n_var,window_size); % Memory allocation with zeros
error_controller_estimation = zeros(1,window_size);% Memory allocation with zeros

for m=1:window_size% for all temporal sliding window
    
    command_signal = 0.0; % initialization
    
    %% Obtain variable \psi (eq. 11) for temporal sliding window
    psi = Psi( data_window(1:end-2,m), x );
    
    k = 1;
    
    for j=1:n_var % for all input variables
        
        n_rules = size( x(j).Rules,1 );% number of rules of input variable j
                
        local_error_variable = 0;% initialization

        for i = 1:n_rules % for all rules of input variable j,
            
            command_signal = command_signal + psi(k)*x(j).Rules(i,4); % Obtain the estimated controller signal using the current fuzzy controller
            
            local_error_variable = local_error_variable + abs(data_window(end,m)*psi(k) - psi(k)*x(j).Rules(i,4)); % Estimation control error of rule i of variable j
            
            k = k +1;
        end

        error_controller_estimation_local(j,m) = local_error_variable; % Save the estimation control error of each input variable
  
    end
    
    error_controller_estimation(m) = data_window(end,m) - command_signal; % estimated control error for all window
    
end

end